# Entrypoints

* `xapi-clusterd` daemon
* `xapi-clusterd-test` (faketest) - unit test which uses a mock implementation of corosync
* `realtest` - implementation of the unit test with the real corosync (spawns VMs, runs the test on real XS)

# Code layout

 - [bin/main.ml](bin/main.ml) - xcli - mostly glue code, new APIs added automatically by idl
 - [bugtool](bugtool/) - defines what info the `xen-bugtool` collects
 - [doc](doc/) - generates html API docs
 - [daemon/main.ml](daemon/main.ml) → xapi-clusterd
    - message-switch service - "Local API" interface that XAPI uses to talk xapi-clusterd
    - port 8896 - "Remote API" xapi-clusterd ↔ xapi-clusterd on different hosts
    - port 8895 - used by the test controller ("realtest")
    - all HTTP calls go through 'handler'
    - whenever you define a new API you need to bind it in main.ml (see S. and RS. calls) S - for local API RS - for remote; also need to modify host_setup in fake.ml
    - `on_startup` - currently asynchronous, but should be synchronous
 - [idl](idl/) - just the test API, and the remote API (i.e. private API),  the XAPI API is in xcp-idl interface
 - [lib](lib/) - main implementation
    - `*parser`: for corosync configuration , match runtime config with the declared config
    - [db](lib/db.ml) : single JSON file persistent DB
    - [fake\_signaler](lib/fake_signaller.ml) - impl. of signaler interface for fake tests using just mutex/condition vars
    - [impl\_corosync](lib/impl_corosync.ml)
    - [real\_signaller](lib/real_signaller.ml) - impl. of fistpoints for real tests using files and inotify
    - [systemio](lib/systemio.ml) - I/O helpers
 - [msc](msc/) - documentation, sequence diagram, [rendered mscgen](http://www.uk.xensource.com/~edvint/msc/)
 - [scripts](scripts/) - fence ack script for DLM
 - [test](test/) - common code
    - [fake](test/fake/) - unit test - test functor instantiated for each host TODO: move fake.ml into fake/
         - `wrpc` - wrapper for all the simulated calls (timeouts, network splits, etc.)
    - [fake/unittest](test/fake/unittest.ml) - for small function unit tests
    - [real](test/real/) - impl. and instantiation of test functor using vagrant acting on real XS VMs
            done with virtual XS hosts controlled by vagrant
    - [test.ml](test/test.ml) - functor (implemented by fake/ and real/)
          Fistpoints - allow you to stop at specific points to perform more operations during testing in a deterministic way
    - `*.py` - python test scripts

[Makefile](Makefile)

- `test` ==> unittests
- `retest` ==> reruns the failed tests
- `realtest` => set `VAGRANT` to testarossa path, and `VAGRANT_HOST` to name of physical XS host
 Example of running just one specific test:
```
VAGRANT_HOST=xrtuk-14-04.xenrt.citrite.net VAGRANT=$HOME/testarossa nosetests --with-xunit --xunit-file=nosetests.xml --verbosity=2 --nocapture --with-id test/test_disable.py:DisableTests.test_scenario3_010b
```
- `planex-refresh`: if you change xcp-idl, or clone xcp-idl as subdir
